# TODO(dragondriver): add more default configs to here
class DefaultConfigs:
    WaitTimeDurationWhenLoad = 0.5  # in seconds
    MaxVarCharLengthKey = "max_length"
    MaxVarCharLength = 65535
    EncodeProtocol = 'utf-8'
    IndexName = ""
